<?php
namespace Teknisa\COZ\Services;

use \Setforms2\Shared\ORM\DateTime;
use \Teknisa\Model\Geral\Entities\Lista;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Coz07200DistribuicaoDaRefeicaoService {

    protected $dataSourceManager;
    protected $environment;
    protected $entityManager;
    protected $novoCodigo;

    public function __construct($dataSourceManager, $environment, $entityManager, $novoCodigo) {
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->novoCodigo = $novoCodigo;
    }


    public function iniciaConexao() {
        $this->entityManager->getConnection()->beginTransaction();
    }


    public function realizaCommit() {
        $this->entityManager->getConnection()->commit();
    }

    public function realizaRollBack() {
        $this->entityManager->getConnection()->rollBack();
    }

    public function insereListaServico($nrorg, $nrLista, $cdContador){
        $lista = new Lista();

        $lista->setNrlista($nrLista);
        $lista->setCdlista($cdContador);

        $lista->setNrorg($nrorg);
        $lista->setNrorginclusao($nrorg);
        $lista->setNrorgultatu($nrorg);
        $lista->setDtinclusao(new \DateTime());
        $lista->setDtultatu(new \DateTime());
        $lista->setCdoperinclusao($this->environment->getCdOperador());
        $lista->setCdoperultatu($this->environment->getCdOperador());

        $this->entityManager->persist($lista);
        $this->entityManager->flush();

        return true;
    }


    public function novaLista($nrorg){
        $nrLista = $this->novoCodigo->getCodeByTableName('LISTA', $nrorg);
        return $nrLista;
    }

    public function retornaAgregadasPorCentralGrupo($nrorg, $central, $cdgrupfili){
        $sql = 'SELECT DISTINCT F.CDFILIAL, FF.NMFILIAL
                  FROM FILIAL FF
            INNER JOIN (SELECT CDFILIAGREGA AS CDFILIAL
                          FROM CELULAPROD
                         WHERE CDCENTRAL = :CDCENTRAL
                           AND NRORG = :NRORG
                         UNION ALL
                        SELECT CDFILIAL
                          FROM FILIAL
                         WHERE CDFILCENTPME = :CDCENTRAL
                           AND NRORG = :NRORG
                           AND :CDGRUPFILI = \'T\'
                         UNION ALL
                        SELECT FL.CDFILIAL
                          FROM FILGRUPFILI FL
                         INNER JOIN FILIAL FI ON ((FI.CDFILIAL = FL.CDFILIAL) AND
                                                 (FI.CDFILCENTPME = :CDCENTRAL) AND
                                                 FI.NRORG = :NRORG)
                         WHERE (FL.CDGRUPFILI = :CDGRUPFILI)
                           AND (FL.NRORG = :NRORG)) F ON (F.CDFILIAL = FF.CDFILIAL AND
                                                         FF.NRORG = :NRORG)
                 ORDER BY F.CDFILIAL';


        $params = array(
            'NRORG' => $nrorg,
            'CDGRUPFILI' => $cdgrupfili,
            'CDCENTRAL' => $central,
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function retornaServico($nrorg, $central, $grupfili, $unidade, $grupserv, $rota){

        $sql = "SELECT DISTINCT S.CDFILIAL, 
                S.CDSERVFILI, 
                S.NMSERVICO, 
                S.CDFILIAL || S.CDSERVFILI AS FILISERV,
                S.CDROTATRANS
                  FROM SERVICO S
                 INNER JOIN (SELECT FL.CDFILIAL
                               FROM FILGRUPFILI FL
                              INNER JOIN FILIAL FI 
                                 ON ((FI.CDFILIAL = FL.CDFILIAL) 
                                    AND (FI.CDFILCENTPME = :CDCENTRAL) 
                                    AND FI.NRORG = :NRORG)
                              WHERE (FL.CDGRUPFILI = :CDGRUPFILI)
                                AND ((INSTR(:CDFILIAGREGA, FL.CDFILIAL))<> 0 
                                    OR (:CDFILIAGREGA = 'T'))
                                AND FL.NRORG = :NRORG
                              UNION ALL
                             SELECT CDFILIAL
                               FROM FILIAL
                              WHERE :CDGRUPFILI = 'T'
                                AND CDFILCENTPME = :CDCENTRAL
                                AND ((INSTR(:CDFILIAGREGA, CDFILIAL))<> 0 
                                    OR (:CDFILIAGREGA = 'T'))
                                AND NRORG = :NRORG
                              UNION ALL
                             SELECT CDFILIAGREGA AS CDFILIAL
                               FROM CELULAPROD
                              WHERE CDCENTRAL = :CDCENTRAL
                                AND NRORG = :NRORG) FIL 
                    ON ((S.CDFILIAL = FIL.CDFILIAL) 
                       AND ((INSTR(:CDGRUPSERV ,S.CDGRUPSERV)<>0) 
                       OR (:CDGRUPSERV = 'T')))
                 WHERE S.IDSERVTRAN = 'S'
                   AND S.IDTIPOSERV <> 'M'
                   AND S.IDSERVATIV = 'S'
                   AND ((INSTR(:CDROTATRANS, S.CDROTATRANS)<>0) 
                       OR (:CDROTATRANS = 'T'))
                 ORDER BY S.CDFILIAL, S.CDSERVFILI, S.NMSERVICO";


        $params = array(
            'NRORG' => $nrorg,
            'CDGRUPSERV' => $grupserv,
            'CDGRUPFILI' => $grupfili,
            'CDFILIAGREGA' => $unidade,
            'CDCENTRAL' => $central,
            'CDROTATRANS' => $rota
        );
        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function atualizaDistribuicaoPrato(
        $nrorg, $nrDistCard, $nrembdistcard, $observacao, $idPrincipal, $seqPrato
    ){
        try{
            $data = new DateTime();
            $operador = $this->environment->getCdOperador();
    
           // $this->iniciaConexao();
    
            //Atualiza a OBSERVACAO da embalagem
            $emb = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Embdistcard')
                ->findOneBy(
                    array(
                        "nrorg" => $nrorg,
                        "nrdistcard" => $nrDistCard,
                        "nrembdistcard" => $nrembdistcard
                    )
                );
    
            if($emb){
                $emb->setDsobsetiqueta($observacao);
    
                $emb->setNrorgultatu($nrorg);
                $emb->setCdoperultatu($operador);
                $emb->setDtultatu($data);
    
                $this->entityManager->persist($emb);
                $this->entityManager->flush();
            }
    
            if ($idPrincipal == 'S') {
                //Atualiza se o item for principal
                $itsemb = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Itembdistcard')
                    ->findBy(
                        array(
                            "nrorg" => $nrorg,
                            "nrdistcard" => $nrDistCard,
                            "nrseqpratcar" => $seqPrato,
                            "nrembdistcard" => $nrembdistcard
                        )
                    );
    
                if ($itsemb) {
                    foreach ($itsemb as $it) {
                        $it->setIdprincipal($idPrincipal);
    
                        $emb->setNrorgultatu($nrorg);
                        $emb->setCdoperultatu($operador);
                        $emb->setDtultatu($data);
    
                        $this->entityManager->persist($emb);
                        $this->entityManager->flush();
                    }
                }
            }
    
            //$this->realizaCommit();
        }catch(\Exception $e){
            throw $e;
        }
    }


    public function atualizaDistribuicaoProduto(
        $nrorg, $nrDistProd, $nrembdistprod, $observacao, $idPrincipal, $nrDistCard
    ){
        try{
            $data = new DateTime();
            $operador = $this->environment->getCdOperador();
    
            // $this->iniciaConexao();
    
            //Atualiza a OBSERVACAO da embalagem
            $emb = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Embdistprod')
                ->findOneBy(
                    array(
                        "nrorg" => $nrorg,
                        "nrembdistprod" => $nrembdistprod,
                        "nrdistprod" => $nrDistProd
                    )
                );
    
            if($emb){
                $emb->setDsobsetiqueta($observacao);
    
                $emb->setNrorgultatu($nrorg);
                $emb->setCdoperultatu($operador);
                $emb->setDtultatu($data);
    
                $this->entityManager->persist($emb);
                $this->entityManager->flush();
            }
    
            if ($idPrincipal == 'S') {
                //Atualiza se o item for principal
                $itsemb = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Itembdistprod')
                    ->findBy(
                        array(
                            "nrorg" => $nrorg,
                            "nrembdistprod" => $nrembdistprod,
                            "nrdistprod" => $nrDistProd,
                            "nrproddistprod" => $nrDistCard
                        )
                    );
    
                if ($itsemb) {
                    foreach ($itsemb as $it) {
                        $it->setIdprincipal($idPrincipal);
    
                        $emb->setNrorgultatu($nrorg);
                        $emb->setCdoperultatu($operador);
                        $emb->setDtultatu($data);
    
                        $this->entityManager->persist($emb);
                        $this->entityManager->flush();
                    }
                }
            }
        // $this->realizaCommit();
        }catch(\Exception $e) {
			throw $e;
		}
    }


    public function salvaDistribuicaoPrato(
        $nrorg, $nrDistCard, $nrDistProd, $cliente, $destino, $nrSequencia, $quantidade, $agrupEmb,
        $observacao, $propEmb, $container, $idPrincipal, $qtPrato, $qtExtra, $comensais
    ){
        try{
            $data = new \DateTime();
    
            $operador = $this->environment->getCdOperador();
    
            // $this->iniciaConexao();
    
            $nrembdistcard = null;
            if ($propEmb == "I") {
                /** @var \Planejamento\Entities\Embdistcard $emb */
                $emb = self::verificarItemPorEmbalagemPrato(
                    $nrorg, $destino, $cliente, $nrDistProd, $agrupEmb, $nrDistCard,
                    $container
                );
    
                if ($emb) {
                    $nrembdistcard = $emb[0]['NREMBDISTCARD'];
                }
            }
    
            if (!$nrembdistcard) {
                $nrembdistcard = $this->novoCodigo->getCodeByTableName(
                    'EMBDISTCARD', $nrorg
                );
    
                /** @var \Planejamento\Entities\Embdistcard $emb */
                $emb = new \Teknisa\Model\Planejamento\Entities\Embdistcard();
    
                $emb->setNrdistcard($nrDistCard);
                $emb->setNrembdistcard($nrembdistcard);
                $emb->setCdcontainer($container);
                $emb->setQtcontainer($quantidade);
                $emb->setCdagrupemb($agrupEmb);
                $emb->setDsobsetiqueta($observacao);
                $emb->setIdcadauto('N');
    
                //Auditoria inclusao
                $emb->setNrorg($nrorg);
                $emb->setNrorginclusao($nrorg);
                $emb->setCdoperinclusao($operador);
                $emb->setDtinclusao($data);
    
                //Auditoria alteração
                $emb->setNrorgultatu($nrorg);
                $emb->setCdoperultatu($operador);
                $emb->setDtultatu($data);
    
                $this->entityManager->persist($emb);
                $this->entityManager->flush();
    
                //Ao menos um prato deve ser principal.
                $idPrincipal = 'S';
            }
    
            $it = new \Teknisa\Model\Planejamento\Entities\Itembdistcard();
    
            $it->setNrdistcard($nrDistCard);
            $it->setNrembdistcard($nrembdistcard);
            $it->setNrseqpratcar($nrSequencia);
            $it->setQtcomensais($comensais);
            $it->setQtprato($qtPrato);
            $it->setQtextra($qtExtra);
            $it->setIdcadauto('N');
            $it->setIdprincipal($idPrincipal);
    
            //Auditoria
            $it->setNrorg($nrorg);
            $it->setNrorginclusao($nrorg);
            $it->setNrorgultatu($nrorg);
            $it->setCdoperinclusao($operador);
            $it->setCdoperultatu($operador);
            $it->setDtinclusao($data);
            $it->setDtultatu($data);
            $it->setIdativo('S');
    
            $this->entityManager->persist($it);
            $this->entityManager->flush();
    
            // $this->realizaCommit();
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function salvaDistribuicaoProduto(
        $nrorg, $filial, $servico, $cliente, $destino, $nrDistProd, $quantidade, $agrupEmb, $observacao,
        $embalagem, $idPrincipal, $qtProduto, $comensais, $nrDistCard
    ){
        try{
            $data = new \DateTime();
            $operador = $this->environment->getCdOperador();
    
            // $this->iniciaConexao();
    
            if ($agrupEmb) {
                /** @var \Planejamento\Entities\Embdistprod $emb */
                $emb = self::verificarItemPorEmbalagemProduto(
                    $nrorg, $filial, $servico, $cliente, $destino, $agrupEmb, $nrDistProd, $embalagem
                );
            } else {
                $emb = null;
            }
    
            if ($emb) {
                $nrembdistprod = $emb[0]['NREMBDISTPROD'];
            } else {
                $nrembdistprod = null;
            }
    
            if (!$nrembdistprod) {
                $nrembdistprod = $this->novoCodigo->getCodeByTableName(
                    'EMBDISTPROD', $nrorg
                );
    
                /** @var \Planejamento\Entities\Embdistprod $emb */
                $emb = new \Teknisa\Model\Planejamento\Entities\Embdistprod();
                $emb->setNrdistprod($nrDistProd);
                $emb->setNrembdistprod($nrembdistprod);
                $emb->setCdcontainer($embalagem);
                $emb->setQtcontainer($quantidade);
                $emb->setCdagrupemb($agrupEmb);
                $emb->setDsobsetiqueta($observacao);
    
                //Auditoria inclusao
                $emb->setIdativo('S');
                $emb->setNrorg($nrorg);
                $emb->setNrorginclusao($nrorg);
                $emb->setCdoperinclusao($operador);
                $emb->setDtinclusao($data);
    
                //Auditoria alteração
                $emb->setNrorgultatu($nrorg);
                $emb->setCdoperultatu($operador);
                $emb->setDtultatu($data);
    
                $this->entityManager->persist($emb);
                $this->entityManager->flush();
    
                //Ao menos um produto deve ser principla
                $idPrincipal = 'S';
            }
    
            $it = new \Teknisa\Model\Planejamento\Entities\Itembdistprod();
    
            /*$nritembdistprod = $this->novoCodigo->getCodeByTableName(
                'ITEMBDISTPROD', $nrorg
            );*/
    
            $it->setNrproddistprod($nrDistCard);
            $it->setNrembdistprod($nrembdistprod);
            $it->setNrdistprod($nrDistProd);
            $it->setQtcomensais($comensais);
            $it->setQtproduto($qtProduto);
            $it->setIdprincipal($idPrincipal);
    
            //Auditoria
            $it->setNrorg($nrorg);
            $it->setNrorginclusao($nrorg);
            $it->setNrorgultatu($nrorg);
            $it->setCdoperinclusao($operador);
            $it->setCdoperultatu($operador);
            $it->setDtinclusao($data);
            $it->setDtultatu($data);
            $it->setIdativo('S');
    
            $this->entityManager->persist($it);
            $this->entityManager->flush();
    
            // $this->realizaCommit();
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificarItemPorEmbalagemPrato(
        $nrorg, $destino, $cliente, $distprod, $agrupamento, $nrDistCard, $container
    ){
        $sql = 'SELECT  EM.*
                  FROM EMBDISTCARD EM,
                       CARDDISTPROD C,
                       DISTCARD DC
                 WHERE DC.NRORG = :P_NRORG
                   AND DC.NRDISTCARD = :P_NRDISTCARD
                   AND (DC.CDDESTFILI = :P_DESTFILI OR :P_DESTFILI IS NULL)
                   AND (DC.CDCLIENTE = :P_CDCLIENTE OR :P_CDCLIENTE IS NULL)
                   AND DC.NRORG = C.NRORG
                   AND C.NRDISTPROD = :P_NRDISTPROD
                   AND C.NRDISTCARD = DC.NRDISTCARD
                   AND UPPER(EM.CDAGRUPEMB) = UPPER(:P_AGRUPAMENTO)
                   AND EM.CDCONTAINER = :P_CDCONTAINER
                   AND EM.NRORG = DC.NRORG
                   AND EM.NRDISTCARD = DC.NRDISTCARD';


        $params = array(
            'P_NRORG' => $nrorg,
            'P_NRDISTCARD' => $nrDistCard,
            'P_DESTFILI' => $destino,
            'P_CDCLIENTE' => $cliente,
            'P_NRDISTPROD' => $distprod,
            'P_AGRUPAMENTO' => $agrupamento,
            'P_CDCONTAINER' => str_pad($container, 5, 0)
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificarItemPorEmbalagemProduto($nrorg, $filial, $servico, $cliente, $destino, $agrupamento, $nrDistProd, $container){
        $sql = 'SELECT IT.*
                  FROM ITEMBDISTPROD IT, EMBDISTPROD EM, PRODDISTPROD PD
                 WHERE IT.IDPRINCIPAL = \'S\'
                   AND IT.NRDISTPROD = :P_NRDISTPROD
                   AND IT.NRORG = :P_NRORG
                   AND UPPER(EM.CDAGRUPEMB) = UPPER(:P_AGRUPAMENTO)
                   AND EM.CDCONTAINER = :P_CDCONTAINER
                   AND EM.NREMBDISTPROD = IT.NREMBDISTPROD
                   AND EM.NRDISTPROD = IT.NRDISTPROD
                   AND EM.NRORG = IT.NRORG
                   AND PD.CDFILIAL = :CDFILIAL
                   AND PD.CDSERVFILI = :CDSERVFILI
                   AND (PD.CDDESTFILI = :CDDESTFILI OR PD.CDDESTFILI IS NULL)
                   AND (PD.CDCLIENTE = :CDCLIENTE OR PD.CDCLIENTE IS NULL)
                   AND PD.NRDISTPROD = EM.NRDISTPROD
                   AND PD.NRPRODDISTPROD = IT.NRPRODDISTPROD
                   AND PD.NRORG = EM.NRORG';

        $params = array(
            'P_NRORG' => $nrorg,
            'P_NRDISTPROD' => $nrDistProd,
            'P_AGRUPAMENTO' => $agrupamento,
            'P_CDCONTAINER' => str_pad($container, 5, 0),
            'CDFILIAL' => $filial,
            'CDSERVFILI' => $servico,
            'CDDESTFILI' => $destino,
            'CDCLIENTE' => $cliente
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function excluiDistribuicaoPrato($nrorg, $nrEmbDistCard, $sequencia, $nrdistcard){
        /** @var \Planejamento\Entities\Itembdistcard $it */
        $itsemb = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Itembdistcard')
            ->findBy(
                array(
                    "nrorg" => $nrorg,
                    "nrembdistcard" => $nrEmbDistCard,
                    "nrdistcard" => $nrdistcard
                )
            );

        if ($itsemb) {
            $this->iniciaConexao();
            foreach($itsemb as $it){
                if (count($itsemb) == 1) {
                    $this->entityManager->remove($it);
                    $this->entityManager->flush();

                    //exclui relacao embalagem
                    $emb = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Embdistcard')
                        ->findOneBy(
                            array(
                                "nrorg" => $nrorg,
                                "nrembdistcard" => $nrEmbDistCard
                            )
                        );

                    foreach($emb as $e){
                        $this->entityManager->remove($e);
                        $this->entityManager->flush();
                    }

                    break;
                } else {
                    if ($it->getNrseqpratcar() == $sequencia and $it->getIdprincipal() == 'S') {
                        return false;
                    } else if ($it->getNrseqpratcar() == $sequencia and $it->getIdprincipal() == 'N') {
                        $this->entityManager->remove($it);
                        $this->entityManager->flush();

                        break;
                    }
                }
            }

            $this->realizaCommit();
            return true;
        }
    }


    public function excluiDistribuicaoProduto($nrorg, $nrEmbDistProduto, $nrdistprod, $nrDistCard){
        /** @var \Planejamento\Entities\Itembdistprod $it */
        $itsemb = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Itembdistprod')
            ->findBy(
                array(
                    "nrorg" => $nrorg,
                    "nrembdistprod" => $nrEmbDistProduto,
                    "nrdistprod" => $nrdistprod,
//                    "nrproddistprod" => $nrDistCard
                )
            );
        if ($itsemb) {
            $this->iniciaConexao();
            if (count($itsemb) == 1) {
                $this->entityManager->remove($itsemb[0]);
                $this->entityManager->flush();

                //exclui relacao embalagem
                $emb = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Embdistprod')
                    ->findOneBy(
                        array(
                            "nrorg" => $nrorg,
                            "nrembdistprod" => $nrEmbDistProduto,
                            "nrdistprod" => $nrdistprod
                        )
                    );

                $this->entityManager->remove($emb);
                $this->entityManager->flush();
            }
            else {
                foreach($itsemb as $it){
                    if ($it->getNrproddistprod() === $nrDistCard) {
                        if ($it->getIdprincipal() == 'S') {
                            return false;
                        } else {
                            $this->entityManager->remove($it);
                            $this->entityManager->flush();
                        }
                    }
                }
            }

            $this->realizaCommit();
            return true;
        }
    }


    public function excluiLista($nrorg, $nrLista)
    {
        $this->iniciaConexao();

        $lista = $this->entityManager->getRepository(
            '\Teknisa\Model\Geral\Entities\Lista'
        )
            ->findBy(
                array(
                    "nrorg" => $nrorg,
                    "nrlista" => $nrLista
                )
            );

        if ($lista) {
            foreach ($lista as $it) {
                $this->entityManager->remove($it);
                $this->entityManager->flush();
            }
        }

        $this->realizaCommit();
    }

    public function verificaPratoPrincipal($nrorg, $nrdistcard, $nrembdistcard){
        /** @var \Planejamento\Entities\Itembdistcard $it */
        $itsemb = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Itembdistcard')
            ->findBy(
                array(
                    "nrorg" => $nrorg,
                    "nrdistcard" => $nrdistcard,
                    "nrembdistcard" => $nrembdistcard
                )
            );
        foreach ($itsemb as $it) {
            if ($it->getIdprincipal() == 'S') {
                return $it;
            }
        }

        return false;
    }

    public function verificaProdutoPrincipal($nrorg, $nrDistProd, $filial, $servico, $cliente, $destino){
        $sql = 'SELECT IT.*
                  FROM ITEMBDISTPROD IT, EMBDISTPROD EM, PRODDISTPROD PD
                 WHERE IT.IDPRINCIPAL = \'S\'
                   AND IT.NRDISTPROD = :NRDISTPROD
                   AND IT.NRORG = :NRORG
                   AND EM.NREMBDISTPROD = IT.NREMBDISTPROD
                   AND EM.NRDISTPROD = IT.NRDISTPROD
                   AND EM.NRORG = IT.NRORG
                   AND PD.CDFILIAL = :CDFILIAL
                   AND PD.CDSERVFILI = :CDSERVFILI
                   AND (PD.CDDESTFILI = :CDDESTFILI OR PD.CDDESTFILI IS NULL)
                   AND (PD.CDCLIENTE = :CDCLIENTE OR PD.CDCLIENTE IS NULL)
                   AND PD.NRDISTPROD = EM.NRDISTPROD
                   AND PD.NRPRODDISTPROD = IT.NRPRODDISTPROD
                   AND PD.NRORG = EM.NRORG';

        $params = array(
            'NRORG' => $nrorg,
            'NRDISTPROD' => $nrDistProd,
            'CDFILIAL' => $filial,
            'CDSERVFILI' => $servico,
            'CDDESTFILI' => $destino,
            'CDCLIENTE' => $cliente
        );

        $item = $this->entityManager->getConnection()->fetchAll($sql, $params);

        if (count($item) > 0) {
            return $item;
        } else {
            return false;
        }
    }

    public function atualizaPratoPrincipal($itemEmb, $nrorg){
        $data = new \DateTime();
        $operador = $this->environment->getCdOperador();
        $itemEmb->setIdprincipal('N');

        //Auditoria
        $itemEmb->setNrorgultatu($nrorg);
        $itemEmb->setCdoperultatu($operador);
        $itemEmb->setDtultatu($data);
        
        
        $this->entityManager->persist($itemEmb);
        $this->entityManager->flush();
        
    }

    public function atualizaProdutoPrincipal($itens, $nrorg){
        /** @var \Teknisa\Model\Planejamento\Entities\Itembdistprod $item */
        $item = $this->entityManager->getRepository(
            '\Teknisa\Model\Planejamento\Entities\Itembdistprod'
        )->findBy(
            array(
                "nrorg" => $itens['NRORG'],
                "nrdistprod" => $itens['NRDISTPROD'],
                "nrembdistprod" => $itens['NREMBDISTPROD'],
                "nrproddistprod" => $itens['NRPRODDISTPROD']
            )
        );

        $itemEmb = $item[0];

        $data = new \DateTime();
        $operador = $this->environment->getCdOperador();

        $itemEmb->setIdprincipal('N');

        //Auditoria
        $itemEmb->setNrorgultatu($nrorg);
        $itemEmb->setCdoperultatu($operador);
        $itemEmb->setDtultatu($data);

        $this->entityManager->persist($itemEmb);
        $this->entityManager->flush();
    }

    public function deletaCodigoControle($nrorg, $nrcontrole){
        $sql = "DELETE FROM CODIGO_CONTROLE
                WHERE NRORG = $nrorg
                AND NRCONTROLE = $nrcontrole";


        $params = array(
            'P_NRORG' => $nrorg,
            'P_NRCONTROLE' => $nrcontrole
        );

        try {
            $this->entityManager->getConnection()->exec($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function atualizaQuantidadeProduto (
        $nrorg, $cdfilial, $nrdistprod, $nrdistcard, $cdservfili, $idtipoprod, $dtcardapio,
        $cdcliente, $cddestfili, $cdproduto, $qttotproduto){

        if ($qttotproduto < 0) $qttotproduto = 0;

        $sql = "UPDATE PRODDISTPROD
                SET QTTOTPRODUTO = :P_QTTOTPRODUTO,
                    QTPORPESSOA = ROUND(:P_QTTOTPRODUTO/NVL(DECODE(QTTOTCOMENSAIS, 0, 1),1),3)
                WHERE NRORG = :P_NRORG AND
                      CDFILIAL = :P_CDFILIAL AND
                      NRDISTPROD = :P_NRDISTPROD AND
                      NRPRODDISTPROD = :P_NRDISTCARD AND
                      IDTIPOPROD = :P_IDTIPOPROD AND
                      CDSERVFILI = :P_CDSERVFILI AND
                      DTCARDAPIO = :P_DTCARDAPIO AND
                      (CDCLIENTE  = NVL(:P_CDCLIENTE,'XXX') OR :P_CDCLIENTE IS NULL) AND
                      (CDDESTFILI = NVL(:P_CDDESTFILI,'XXX') OR :P_CDDESTFILI IS NULL) AND
                      CDPRODUTO = :P_CDPRODUTO";

        $params = array(
            'P_NRORG' => $nrorg,
            'P_CDFILIAL' => $cdfilial,
            'P_NRDISTPROD' => $nrdistprod,
            'P_NRDISTCARD' => $nrdistcard,
            'P_CDSERVFILI' => $cdservfili,
            'P_IDTIPOPROD' => $idtipoprod,
            'P_DTCARDAPIO' => $dtcardapio,
            'P_CDCLIENTE' => $cdcliente,
            'P_CDDESTFILI' => $cddestfili,
            'P_CDPRODUTO' => $cdproduto,
            'P_QTTOTPRODUTO' => $qttotproduto
        );
        try {
            $this->entityManager->getConnection()->executeUpdate($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function atualizaQuantidadePrato (
        $nrorg, $nrdistcard, $cdproduto, $qttotproduto
    ){	
        $total = 0.0;
        $novaQtde = array();
        $keyMaxQt = 0;
        $rows = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Cnfdistcard')
            ->findBy(
                array(
                    "cdprodproducao" => $cdproduto,
                    "nrdistcard" => $nrdistcard,
                    "nrorg" => $nrorg
                )
            );
        foreach ($rows as $key => $row) {
            $total += $row->getQttotprato();
            if($row->getQttotprato() > $keyMaxQt){
                $keyMaxQt = $key;
            }
        }
        if ($total == 0) $total = 1;

        foreach ($rows as $key => $row) {
            if($row->getQttotprato() == 0) $perc = 100;
            else $perc = ($row->getQttotprato() + $row->getQttotextra()) * 100
                    / $total;
                    
            array_push($novaQtde, $qttotproduto * $perc / 100);
        }

        if(array_sum($novaQtde) > $qttotproduto){
            $novaQtde[$keyMaxQt] += array_sum($novaQtde) - $qttotproduto;
        } else if(array_sum($novaQtde) < $qttotproduto){
            $novaQtde[$keyMaxQt] -= $qttotproduto - array_sum($novaQtde);
        }

        foreach ($rows as $key => $row) {
            $qtPrato = round($novaQtde[$key], 5);
            if ($qtPrato < 0) $qtPrato = 0;

            $row->setQttotextra(0);
            $row->setQttotprato($qtPrato);
            $row->setQtporpessoa(round($qtPrato/$row->getQttotcomensais(), 5));
            $this->entityManager->persist($row);
            $this->entityManager->flush();
        }
    }

    public function atualizaDitribuicaoRollback($nrdistprod, $nrorg){
        $distprod = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Distprod')
            ->findOneBy(
                array(
                    "nrdistprod" => $nrdistprod,
                    "nrorg" => $nrorg
                )
            );

        if($distprod){
            $distprod->setIdgerarequi('N');

            $this->entityManager->persist($distprod);
            $this->entityManager->flush();
        }
    }

    /** Função responsável pela execução da procedure de distribuição da produção
     */
    public function DistribuiProducao(
        $operacao, $cdcentral, $nrdistprod,
        $dtini, $dtfin, $nrlistaserv, $nrorg, $cdoperador,
        $consumo
    )
    {
        try {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection,'DISTRIBUI_PRODUCAO_COZ');
            $procedure->addParam(
                new Param (
                    'P_OPERACAO',
                    Param::PARAM_INPUT,
                    $operacao,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param (
                    'P_CDCENTRAL',
                    Param::PARAM_INPUT,
                    $cdcentral,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param (
                    'P_NRDISTPROD',
                    Param::PARAM_INPUT,
                    $nrdistprod,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param (
                    'P_DTINI',
                    Param::PARAM_INPUT,
                    $dtini,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param (
                    'P_DTFIN',
                    Param::PARAM_INPUT,
                    $dtfin,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_NRLISTA_SERV',
                    Param::PARAM_INPUT,
                    $nrlistaserv,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_CDOPERADOR',
                    Param::PARAM_INPUT,
                    $cdoperador,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_NRORG',
                    Param::PARAM_INPUT,
                    $nrorg,
                    Param::PARAM_TYPE_STR
                )
            );
           
            $procedure->addParam(
                new Param (
                    'P_RETORNO',
                    Param::PARAM_OUTPUT,
                    NULL,
                    Param::PARAM_TYPE_STR,
                    400
                )
            );
			
            $procedure->addParam(
                new Param (
                    'P_CONSUMO',
                    Param::PARAM_INPUT,
                    $consumo,
                    Param::PARAM_TYPE_STR
                )
            );


            return $procedure->execute();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /** Função responsável pela execução da procedure responsável pela geração das notas fiscais e retiradas
     * envolvidas na distribuição da refeição
     */

    public function EfetivaDistribuicao(
        $nrdistprod, $operacao, $cdfiliproc, $cdoperador, $dataproc,
        $dtretirada, $dtcardapio, $nrorg, $nrlistaprod, $nrcontrole
    ){
        try {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection,'EFETIVA_DISTRIBUICAO');

            $procedure->addParam(new Param ('P_NRDISTPROD',Param::PARAM_INPUT,$nrdistprod,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_OPERACAO',Param::PARAM_INPUT,$operacao,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_CDFILIPROC',Param::PARAM_INPUT,$cdfiliproc,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_CDOPERADOR',Param::PARAM_INPUT,$cdoperador,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_DATAPROC',Param::PARAM_INPUT,$dataproc,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_DTRETIRADA',Param::PARAM_INPUT,$dtretirada,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_DTCARDAPIO',Param::PARAM_INPUT,$dtcardapio,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_NRORG',Param::PARAM_INPUT,$nrorg,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_NRCONTROLE',Param::PARAM_INPUT,$nrcontrole,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_RETORNO',Param::PARAM_OUTPUT,NULL,Param::PARAM_TYPE_STR,400));
            $procedure->addParam(new Param ('P_NRLISTA_PROD',Param::PARAM_INPUT,$nrlistaprod,Param::PARAM_TYPE_STR));
            return $procedure->execute();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificarNota($nrlista, $nrdistprod){
		$sql = "SELECT DISTINCT PROD_JA_EFET.CDPRODUTO
				  FROM 
						(SELECT P.NRRETIRADA, P.NRLANCESTQENTR, P.CDPRODUTO
						   FROM PRODDISTPROD P
						  WHERE P.CDFILANCTO IS NOT NULL
							AND P.NRDISTPROD = :NRDISTPROD
									AND P.CDPRODUTO NOT IN (SELECT CDLISTA FROM LISTA WHERE NRLISTA = :NRLISTA)
							
						  UNION ALL
							
						 SELECT CNF.NRRETIRADA, CNF.NRLANCESTQENTR, CNF.CDPRODPRODUCAO AS CDPRODUTO
						   FROM CARDDISTPROD C, CNFDISTCARD CNF
						  WHERE CNF.CDFILANCTO IS NOT NULL
							AND CNF.CDPRODPRODUCAO NOT IN (SELECT CDLISTA FROM LISTA WHERE NRLISTA = :NRLISTA)
							AND C.NRDISTPROD = :NRDISTPROD
							AND C.NRDISTCARD = CNF.NRDISTCARD) PROD_JA_EFET
							
				INNER JOIN (SELECT P.NRRETIRADA, P.NRLANCESTQENTR, P.CDPRODUTO
								FROM PRODDISTPROD P
							   WHERE P.CDFILANCTO IS NOT NULL
								 AND P.NRDISTPROD = :NRDISTPROD
									   AND P.IDEFETIVADA = 'S'
								   AND P.CDPRODUTO IN (SELECT CDLISTA FROM LISTA WHERE NRLISTA = :NRLISTA)
						   
							   UNION ALL
								 
							  SELECT CNF.NRRETIRADA, CNF.NRLANCESTQENTR, CNF.CDPRODPRODUCAO AS CDPRODUTO
								FROM CARDDISTPROD C, CNFDISTCARD CNF
							   WHERE CNF.CDPRODPRODUCAO IN (SELECT CDLISTA FROM LISTA WHERE NRLISTA = :NRLISTA)
								 AND C.NRDISTPROD = :NRDISTPROD
								 AND C.NRDISTCARD = CNF.NRDISTCARD) EFETIVANDO 
										ON (
										/*EFETIVANDO.NRRETIRADA = PROD_JA_EFET.NRRETIRADA OR */
										EFETIVANDO.NRLANCESTQENTR = PROD_JA_EFET.NRLANCESTQENTR)
									   AND EFETIVANDO.CDPRODUTO <> PROD_JA_EFET.CDPRODUTO";

        $params = array(
            'NRLISTA'    => $nrlista,
            'NRDISTPROD' => $nrdistprod
        );
        
        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function CalculaEmbalagens($nrdistprod, $nrorg){
        try {
            $connection = $this->entityManager->getConnection();
            
            $procedure = new StoredProcedure($connection,'CALCULA_EMB_SERV_COZ');

            $procedure->addParam(new Param ('P_NRDISTPROD',Param::PARAM_INPUT,$nrdistprod,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_NRORG',Param::PARAM_INPUT,$nrorg,Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param ('P_RETORNO', Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 20));

            return $procedure->execute();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /** Função responsável pela execução da procedure responsável pela geração das notas fiscais e retiradas
     * envolvidas na distribuição da refeição
     */

    public function ControleNovoCodigo($nrdistprod, $cdfiliproc, $nrorg){
        try {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection,'CONTROLE_NOVO_CODIGO');

            $procedure->addParam(
                new Param (
                    'P_NRDISTPROD',
                    Param::PARAM_INPUT,
                    $nrdistprod,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_CDFILIPROC',
                    Param::PARAM_INPUT,
                    $cdfiliproc,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_NRORG',
                    Param::PARAM_INPUT,
                    $nrorg,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_NRCONTROLE',
                    Param::PARAM_OUTPUT,
                    NULL,
                    Param::PARAM_TYPE_STR,
                    20
                )
            );

            return $procedure->execute();

        } catch (\Exception $e) {
            throw $e;
        }
    }
}